<?php
//
// +----------------------------------------------------------------------+
// | PHP Version 5                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 3.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/3_0.txt.                                  |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Hartmut Holzgraefe <hholzgra@php.net>                       |
// +----------------------------------------------------------------------+
//
// $Id: Element.php,v 1.1 2004/04/01 18:43:26 hholzgra Exp $
//

/**
* Abstract base class for all PHP code elements
*
* @version $Id: Element.php,v 1.1 2004/04/01 18:43:26 hholzgra Exp $
*/

abstract class PECL_Gen_Element 
  extends PEAR 
{

    /**
     * Checks whether a string is a valid PHP type name and returns the official name
     *
     * @access public
     * @param  string Type name
     * @return string The official type name or boolean false if not a type
     */
    function is_type($name) 
    {
        static $types = array("void"     => "void", 
                              "bool"     => "bool", 
                              "boolean"  => "bool", 
                              "int"      => "int", 
                              "integer"  => "int", 
                              "float"    => "float", 
                              "double"   => "float", 
                              "real"     => "float", 
                              "string"   => "string", 
                              "array"    => "array",
                              "object"   => "object",
                              "resource" => "resource",
                              "mixed"    => "mixed",
                              "callback" => "callback",
                              "stream"   => "stream"
                              );
        
        if (isset($types[$name])) {
            return $types[$name];
        } else {
            return false;
        }
    }


    /**
     * Checks whether a string is a reserved name
     *
     * @access public
     * @param  string name
     * @return bool   true if reserved
     */
    function is_keyword($name) 
    {
        // these are taken from zend_language_scanner.l
        static $reserved = array(
                                 "abstract",
                                 "and",
                                 "array",
                                 "as",
                                 "break", 
                                 "case",
                                 "catch",
                                 "class",
                                 "const",
                                 "continue",
                                 "declare",
                                 "default",
                                 "die",
                                 "do",
                                 "echo",
                                 "else", 
                                 "elseif",
                                 "empty",
                                 "enddeclare",
                                 "endfor",
                                 "endforeach",
                                 "endif",
                                 "endwhile",
                                 "eval",
                                 "exit",
                                 "extends",
                                 "final",
                                 "for", 
                                 "foreach", 
                                 "function", 
                                 "global",
                                 "if", 
                                 "implements",
                                 "include",
                                 "include_once",
                                 "instanceof",
                                 "interface",
                                 "isset",
                                 "list",
                                 "new",
                                 "or",
                                 "print",
                                 "private", 
                                 "protected", 
                                 "public", 
                                 "require",
                                 "require_once",
                                 "return", 
                                 "static",
                                 "throw",
                                 "try",
                                 "unset",
                                 "unset",
                                 "use",
                                 "var",
                                 "while", 
                                 "xor",
                                 );
        
        foreach ($reserved as $keyword) {
            if (!strcasecmp($keyword, $name)) {
                return true;
            }
        }

        return false;
    }



    /**
     * Checks whether a string is a valid PHP name
     *
     * @access public
     * @param  string The name to check
     * @return bool   true for valid names, false otherwise
     */
    function is_name($name) 
    {
        if (ereg("^[[:alpha:]_][[:alnum:]_]*$",$name)) {
            // TODO reserved words
            return true;
        } 
        return false;
    }

    /**
     * Generate C code for element
     *
     * @access  public
     * @param   string Extension name
     * @return  string C code 
     */
    abstract function c_code($name);

    /**
     * Generate C code header block for all elements of this class
     *
     * @access public
     * @param  string Extension name
     * @return string C code
     */
    static function c_code_header($name) 
    {
        return "";
    }

    /**
     * Generate C code footer block for all elements of this class
     *
     * @access public
     * @param  string Extension name
     * @return string C code
     */
    static function c_code_footer($name) 
    {
        return "";
    }

    /**
     * Generate C include file definitions for element
     *
     * @access  public
     * @return  string C header code 
     */
    function h_code() 
    {
        return "";
    }

    /**
     * Generate Editor settings block for C source files
     *
     * @access public
     * @return string Editor settings comment block
    */
    static function c_code_editor_settings() 
  {
            return '
/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
';
    }


    /**
     * Generate DocBook documentation code for element
     *
     * @access  public
     * @param  string id basename for extension
     * @return  string DocBook XML content
     */
    abstract function docbook_xml($extension);

    /**
     * Generate DocBook XML header block for all elements of this class
     *
     * @access public
     * @param  string Extension name
     * @return string DocBook XML fragment
     */
    static function docbook_xml_header($name) 
    {
        return "";
    }

    /**
     * Generate DocBook XML footer block for all elements of this class
     *
     * @access public
     * @param  string Extension name
     * @return string DocBook XML fragment
     */
    static function docbook_xml_footer($name) 
    {
        return "";
    }

    /**
     * Generate Editor settings block for documentation files
     *
     * @access public
     * @param  int    Directory nesting depth of target file (default: 3)
     * @return string Editor settings comment block
    */
    static function docbook_editor_settings($level=3) 
    {
        return '
<!-- Keep this comment at the end of the file
Local'.' variables:
mode: sgml
sgml-omittag:t
sgml-shorttag:t
sgml-minimize-attributes:nil
sgml-always-quote-attributes:t
sgml-indent-step:1
sgml-indent-data:t
indent-tabs-mode:nil
sgml-parent-document:nil
sgml-default-dtd-file:"'.str_repeat("../",$level).'manual.ced"
sgml-exposed-tags:nil
sgml-local-catalogs:nil
sgml-local-ecat-files:nil
End:
vim600: syn=xml fen fdm=syntax fdl=2 si
vim: et tw=78 syn=sgml
vi: ts=1 sw=1
-->
';
    }

}

?>
