<?php
//
// +----------------------------------------------------------------------+
// | PHP Version 5                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 3.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/3_0.txt.                                  |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Hartmut Holzgraefe <hholzgra@php.net>                       |
// +----------------------------------------------------------------------+
//
// $Id: Function.php,v 1.8 2004/11/11 15:10:29 hholzgra Exp $
//

    require_once "PECL/Gen/Tools/Indent.php";

    /**
     * Class describing a function within a PECL extension 
     *
     * @access public
     */
    
    class PECL_Gen_Element_Function 
      extends PECL_Gen_Element 
    {
        /**
         * The function name
         *
         * @access private
         * @var     string
         */
        var $name  = "unknown";

        function set_name($name) 
        {
            if (!self::is_name($name)) {
                return $this->raiseError("'$name' is not a valid function name");
            }
            
            switch ($this->role) {
            case "internal":
                $err = $this->is_internal_name($name);
                if (PEAR::isError($err)) {
                    return $err;
                }
                break;

            case "public":
                // keywords are not as function names
                if (self::is_keyword($name)) {
                    return $this->raiseError("'$name' is a reserved word which is not valid for function names");
                }
                // you should not redefine standard PHP functions
                foreach(get_extension_funcs("standard") as $stdfunc) {
                    if (!strcasecmp($name, $stdfunc)) {
                        return $this->raiseError("'$name' is already the name of a PHP standard function");
                    }
                }
                break;
            }

            $this->name = $name;
            
            return true;
        }




        /**
         * A short description
         *
         * @access private
         * @var     string
         */
        var $summary = "";

        function set_summary($text)
        {
            $this->summary = $text;
            return true;
        }




        /**
         * A long description
         *
         * @access private
         * @var     string
         */
        var $description  = "";

        function set_description($text)
        {
            $this->description = $text;
            return true;
        }




        /**
         * Type of function: private, internal, public
         *
         * @access private
         * @var     string
         */
        var $role  = "public";

        function set_role($role)
        {
            if (!in_array($role, array("private", "internal", "public"))) {
                return $this->raiseError("'$role' is not a valid function role"); 
            }
            
            if ($role == "internal") {
                $err = $this->is_internal_name($this->name);
                if (PEAR::isError($err)) {
                    return $err;
                }               
            }

            $this->role = $role;
            
            return true;
        }

        function get_role() 
        {
            return $this->role;
        }




        /**
         * Function prototype
         *
         * @access private
         * @var     string
         */
        var $proto = "void unknown(void)";

        /**
         * Function returntype (parsed from proto)
         *
         * @access private
         * @var     string
         */
        var $returns = "void";

        /**
         * Function parameters (parsed from proto)
         *
         * @access private
         * @var     array
         */
        var $params = array();

        /**
         * Number of optional parameters (parsed from proto)
         *
         * @access private
         * @var     int
         */
        var $optional = 0;

        function set_proto($proto) 
        {
            /* TODO
                 a 'real' parser is needed here (lex/yacc with PHP output anyone?)
                 to support stuff like default values 
                 the current tokenizer is not clever enough for this ...

                 the grammer for a prototype would look like this: ?

                 proto: type name '(' param-spec ')'

                 name: [A-Za-z_][A-Za-z0-9_]*
                 
                 type: "void"
                     | typespec

                 typespec: typename
                         | typename '&'
                         
                 typename: 'bool'  | 'boolean'
                         | 'int'   | 'integer' | 'long'
                         | 'float' | 'double'  | 'real'
                         | 'string'
                         | 'array'
                         | 'object'   | 'object' name
                         | 'resource' | 'resource' name
                         | 'mixed'
                         | 'callback'
                         | 'stream'

				 param-spec: 'void'
                           | '...'
                           | param-list
						   | param-list ',' '...'

                 param-list: parameter
                           | parm-list ',' parameter
                                    
                 parameter: typespec name
                          | typespec name '=' default-value

                 default-value: "true" | "false" | "array()" | "null"
                              | numeric-value
                              | string

                 string: '"' character* '"'
          
                 number: ... the usual int, float, hex and octal stuff ...
             */
            
            // 'tokenize' it
            $tokens = array();

            // we collect valid C names as Strings, any other character for itself, blanks are skipped
            // TODO: this does no longer work if we ever add default values ...
            $len = strlen($proto);
            $name = "";
            for ($n = 0; $n < $len; $n++) {
                $char = $proto{$n};
                if (ctype_alpha($char) || $char == '_' || ($n && ctype_digit($char))) {
                    $name .= $char;
                } else if ($char == '.' && $proto{$n+1} == '.' && $proto{$n+2} == '.') {
                    $name = "...";
					$n += 2;
                } else {
                    if ($name) $tokens[] = $name;
                    $name = "";
                    if (trim($char)) {
                        $tokens[] = $char;
                    }
                }
            }

            if ($name) {
                $tokens[] = $name;
            }

            $n = 0;
            $opts = 0;
			$numopts = 0;

            $params = array();

            $return_type = ($this->is_type($tokens[$n])) ? $tokens[$n++] : "void";

            $function_name = $tokens[$n++];

            if ($return_type === "resource" && $tokens[$n] !== "(") {
                $return_subtype = $function_name;
                $function_name = $tokens[$n++];
            }

            if (! $this->is_name($function_name)) {
                return $this->raiseError("$function_name is not a valid function name");
            }

            if ($function_name != $this->name) {
                if ($this->name === "unknown") {
                    $this->name = $function_name;
                } else {
                    return $this->raiseError("proto function name is '$function_name' instead of '{$this->name}'");
                }
            }

            if ($tokens[$n++] != '(') return $this->raiseError("'(' expected instead of '$tokens[$n]'");
            
            if ($tokens[$n] == ')') { 
                /* done */
            } else if ($tokens[$n] == '...') {
				$params[0]['type'] = "...";
                $n++;
            } else {
                for ($param = 0; $tokens[$n]; $n++, $param++) {
                    if ($tokens[$n] == '[') {
                        $params[$param]['optional'] = true;
                        $opts++;
                        $n++;
                        if ($param > 0) { 
                            if ($tokens[$n] != ',') {
								return $this->raiseError("',' expected after '[' instead of '$token[$n]'");
							}
                            $n++;
                        }
                    }

                    if (!$this->is_type($tokens[$n])) {
						return $this->raiseError("type name expected instead of '$tokens[$n]'");
					}

                    $params[$param]['type'] = $tokens[$n];
                    $n++;

                    if ($tokens[$n] == "&") {
                        $params[$param]['by_ref'] = true;
                        $n++;
                    }

                    if ($this->is_name($tokens[$n])) {
                        $params[$param]['name']=$tokens[$n];
                        $n++;
                    }

                    if ($tokens[$n] == "&") {
                        $params[$param]['by_ref'] = true;
                        $n++;
                    }

                    if ($params[$param]['type'] === "resource" && $this->is_name($tokens[$n])) {
                        $params[$param]['subtype'] = $params[$param]['name'];
                        $params[$param]['name'] = $tokens[$n];
                        $n++;
                    }

                    if ($tokens[$n] == '[') {
                        $n--;
                        continue;
                    }

                    if ($tokens[$n] == ',') continue;
                    if ($tokens[$n] == ']') break;
                    if ($tokens[$n] == ')') break;            

                }

                $numopts = $opts;
                while ($tokens[$n] == ']') {
                    $n++;
                    $opts--;
                }
                if ($opts != 0) {
                    return $this->raiseError("'[' / ']' count mismatch");
                }
            } 

            if ($tokens[$n] != ')') {
                return $this->raiseError("')' expected instead of '$tokens[$n]'");
            }

            $this->name     = $function_name;
            $this->returns  = $return_type;

            if (isset($return_subtype)) {
                $this->returns .= " $return_subtype";
            }

            $this->params   = $params;
            $this->optional = $numopts;

            return true;
        }




        /**
         * Code snippet
         *
         * @access private
         * @var    string
         */
        var $code = "";

        /**
         * Source file of code snippet
         *
         * @access private
         * @var    string
         */
        var $code_file = "";

        /**
         * Source line of code snippet
         *
         * @access private
         * @var    int
         */
        var $code_line = 0;

        function set_code($text, $line = 0, $file = "")
        {
            $this->code = $text;
            $this->code_file = $file;
            $this->code_line = 0;
            return true;
        }




        static function is_internal_name($name)
        {
            if (!in_array($name, array("MINIT", "MSHUTDOWN", "RINIT", "RSHUTDOWN", "MINFO"))) {
                return $this->raiseError("'$name' is not a valid internal function name");
            }

            return true;
        }


        function c_code($extension) 
        {
            $code = "";

            $returns = explode(" ", $this->returns);

            switch ($this->role) {
            case "public":
                // function prototype comment
                $code .= "\n/* {{{ proto {$this->returns} {$this->name}(";
                if (isset($this->params)) {
                    foreach ($this->params as $key => $param) {
                        if (!empty($param['optional'])) {
                            $code .= " [";
                        }
                        if ($key) { 
                            $code .= ", ";
                        }
                        $code .= $param['type'] . " ";
                        if (isset($param['subtype'])) {
                            $code .= $param['subtype']." ";
                        }
                        if ($param['type'] !== 'void' && $param['type'] !== '...') {
                            if (isset($param['by_ref'])) {
                                $code .= "&";
                            }
                            $code .= $param['name']; 
                        }
                    }
                }
                for ($n = $this->optional; $n > 0; $n--) {
                    $code .= "]";
                }
                $code .= ")\n  ";
                if (!empty($this->summary)) {
                    $code .= $this->summary;
                }
                $code .= " */\n";

                // function declaration
                $code .= "PHP_FUNCTION({$this->name})\n";
                $code .= "{\n";
                
                // for functions returning a named resource we create payload pointer variable
                if ($returns[0] === "resource" && isset($returns[1])) {
                    $resource = $extension->resources[$returns[1]];
                    $payload  = $resource->payload;
                    if ($resource->alloc === "yes") {
                        $code .= "    $payload * return_res = ($payload *)emalloc(sizeof($payload));\n";
                    } else {
                        $code .= "    $payload * return_res;\n";
                    }
                }

                // now we create local variables for all parameters
                // at the same time we put together the parameter parsing string
                if (is_array($this->params) && count($this->params)) {
					$arg_string = "";
                    $arg_pointers = array();
                    $optional = false;
                    $res_fetch = "";
                    foreach ($this->params as $key => $param) {
                        if ($param["type"] === "void"  || $param["type"] === "...") {
							continue;
						}

                        $name = $param['name']; 
                        $arg_pointers[] = "&$name";

                        if (isset($param['optional']) && !$optional) {
                            $optional = true;
                            $arg_string .= "|";
                        }

                        switch ($param['type']) {
                        case "bool":
                            $arg_string .= "b";
                            $code .= "    zend_bool $name = 0;\n";
                            break;

                        case "int":
                            $arg_string .= "l";
                            $code .= "    long $name = 0;\n";
                            break;

                        case "float":
                            $arg_string .= "d";
                            $code .= "    double $name = 0.0;\n";
                            break;

                        case "string":
                            $arg_string .= "s";
                            $code .= "    char * $name = NULL;\n";
                            $code .= "    int {$name}_len = 0;\n";
                            $arg_pointers[] = "&{$name}_len";
                            break;

                        case "array":
                            $arg_string .= "a";
                            $code .= "    zval * $name = NULL;\n";
                            break;

                        case "object": 
                            $arg_string .= "o";
                            $code .= "    zval * $name = NULL;\n";
                            break;

                        case "resource":
                            $arg_string .= "r";
                            $code .= "    zval * $name = NULL;\n";
                            $code .= "    int {$name}_id = -1;\n";

                            if (isset($param['subtype'])) {
                                $resource = $extension->resources[$param['subtype']];
                                $varname = "res_{$name}";
                                $code .= "    {$resource->payload} * $varname;\n";
                                
                                $res_fetch .= "    ZEND_FETCH_RESOURCE($varname, {$resource->payload} *, &$name, {$name}_id, \"$param[subtype]\", le_$param[subtype]);\n";
                            } else {
                                $res_fetch .="    ZEND_FETCH_RESOURCE(???, ???, $name, {$name}_id, \"???\", ???_rsrc_id);\n";
                            }
                            break;

                        case "stream":
                            $arg_string .= "r";
                            $code .= "    zval * _z$name = NULL;\n";
                            $code .= "    php_stream * $name = NULL:\n";
                            $res_fetch.= "    php_stream_from_zval($name, &_z$name);\n"; 
                            break;

                        case "mixed":
                        case "callback":
                            $arg_string .= "z";
                            $code .= "    zval * $name = NULL;\n";
                            break;
                        }
                    }
                } 

                // now we do the actual parameter parsing
                if (empty($arg_string)) {
					if (!empty($this->params) && $this->params[0]['type'] == "...") {
						/* custom parsing code required */
					} else {
						$code .= "    if (ZEND_NUM_ARGS()>0)  {\n        WRONG_PARAM_COUNT;\n    }\n\n";
					}
                } else {
                    $code .= "
    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, \"$arg_string\", ".join(", ",$arg_pointers).") == FAILURE) {
        return;
    }

";
                    if ($res_fetch) {
                        $code.="$res_fetch\n\n";
                    }
                }
                    
                // for functions returning an array we initialize return_value
                if ($returns[0] === "array") {
                    
                }

                if ($this->code) {
                    // if function code is specified so we add it here
                    // Work in progress: add preprocessor #line directives
                    // so that compiler error messages refere to the original XML source
                    if ($this->code_line) {
                        echo "#line {$this->code_line}";
                        if ($this->code_file) {
                            echo $this->code_file;
                        }
                        echo "\n";
                    }
                    if ($extension->language == "c") {
                        // in C variable declarations have to be at the very beginning
                        // of a block, so we have to add {...} around the snippet
                        $code .= "    {\n" . PECL_Gen_Tools_Indent::indent(8, $this->code) . "    }\n"; 
                    } else {
                        // in C++ variable may be declared at any time
                        $code .= PECL_Gen_Tools_Indent::indent(4, $this->code)."\n";
                    }

                    // when a function returns a named resource we know what to do
                    if ($returns[0] == "resource" && isset($returns[1])) {
                        $code .= "    ZEND_REGISTER_RESOURCE(return_value, return_res, le_$returns[1]);\n";
                    }
                } else {
                    // no code snippet was given so we produce a suggestion for the return statement
                    $code .= "    php_error(E_WARNING, \"{$this->name}: not yet implemented\"); RETURN_FALSE;\n\n";
                    
                    switch ($returns[0]) {
                    case "void":
                        break;
                        
                    case "bool":
                        $code .= "    RETURN_FALSE;\n"; 
                        break;
                        
                    case "int":
                        $code .= "    RETURN_LONG(0);\n"; 
                        break;
                        
                    case "float":
                        $code .= "    RETURN_DOUBLE(0.0);\n";
                        break;
                        
                    case "string":
                        $code .= "    RETURN_STRINGL(\"\", 0, 1);\n";
                        break;
                        
                    case "array":
                        $code .= "    array_init(return_value);\n";
                        break;
                        
                    case "object": 
                        $code .= "    object_init(return_value)\n";
                        break;
                        
                    case "resource":
                        if (isset($returns[1])) {
                            $code .= "    ZEND_REGISTER_RESOURCE(return_value, return_res, le_$returns[1]);\n";
                        } else {
                            $code .= "    /* RETURN_RESOURCE(...); */\n";
                        }
                        break;
                        
                    case "stream":
                        $code .= "    /* php_stream_to_zval(stream, return_value); */\n";
                        break;
                        
                    case "mixed":
                        $code .= "    /* RETURN_...(...); */\n";              
                        break;
                        
                    default: 
                        $code .= "    /* UNKNOWN RETURN TYPE '$this->returns' */\n";
                        break;
                    }
                }
                
                $code .= "}\n/* }}} */\n\n";
                break;
                
            case "internal":
                if (!empty($this->code)) {
                    $code .= "    {\n";
                    $code .= PECL_Gen_Tools_Indent::indent(8, $this->code."\n");
                    $code .= "    }\n";
                }
                break;
                
            case "private":
                $code .= PECL_Gen_Tools_Indent::indent(0, $this->code."\n");              
                break;
            }
            return $code;
        }
        
        function docbook_xml($base) 
        {
            $xml = 
'<?xml version="1.0" encoding="iso-8859-1"?>
<!-- '.'$'.'Revision: 1.0 $ -->
  <refentry id="function.' . strtolower(str_replace("_", "-", $this->name)) . '">
   <refnamediv>
    <refname>' . $this->name . '</refname>
    <refpurpose>' . $this->summary . '</refpurpose>
   </refnamediv>
   <refsect1>
    <title>Description</title>
     <methodsynopsis>
';
            
            $xml .= "      <type>{$this->returns}</type><methodname>{$this->name}</methodname>\n";
            if (empty($this->params) || $this->params[0]["type"] === "void") {
                $xml .= "      <void/>\n";
            } else if ($this->params[0]["type"] === "...") {
				$xml .= "      <methodparam choice='opt' rep='repeat'><type>mixed</type><parameter>...</parameter></methodparam>\n";
			} else {
                foreach ($this->params as $key => $param) {
                    if (isset($param['optional'])) {
                        $xml .= "      <methodparam choice='opt'>";
                    } else {
                        $xml .= "      <methodparam>";
                    }
                    $xml .= "<type>$param[type]</type><parameter>";
                    if (isset($param['by_ref'])) {
                        $xml .= "&amp;"; 
                    }
                    $xml .= "$param[name]</parameter>";
                    $xml .= "</methodparam>\n";
                }
            }


            $description = $this->description;

            if (!strstr($this->description,"<para>")) {
                $description = "     <para>\n$description     </para>\n";
            }

            $xml .= 
'     </methodsynopsis>
'.$description.'
   </refsect1>
  </refentry>
';
            $xml .= $this->docbook_editor_settings(4);
 
            return $xml;
        }
        
        function write_test($extension) 
        {
            $fp = fopen("{$extension->name}/tests/{$this->name}.phpt", "w");
            fputs($fp, 
"--TEST--
{$this->name}() function
--SKIPIF--
<?php if (!extension_loaded('{$extension->name}')) print 'skip'; ?>
--POST--
--GET--
--FILE--
<?php 
    echo 'no test case for {$this->name}() yet';
?>
--EXPECT--
no test case for {$this->name}() yet");
            fclose($fp);
        }
    }

?>