<?php
//
// +----------------------------------------------------------------------+
// | PHP Version 5                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 3.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/3_0.txt.                                  |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Hartmut Holzgraefe <hholzgra@php.net>                       |
// +----------------------------------------------------------------------+
//
// $Id: Ini.php,v 1.1 2004/04/01 18:43:26 hholzgra Exp $
//

  /**
   * Class describing a PHP ini directive within a PECL extension 
   *
   * @access public
   */
  class PECL_Gen_Element_Ini 
    extends PECL_Gen_Element 
  {
      /**
       * Directive name
       *
       * @access private
       * @var     string
       */
      var $name;
      
      /**
       * Set method for name
       *
       * @access public
       * @var string directive name
       */
      function set_name($name)
      {
          if (!self::is_name($name)) {
              return $this->raiseError("'$name' is not a valid php.ini directive name");
          }
          
          $this->name = $name;
          
          return true;
      }




      /**
       * Directive data type
       *
       * @access private
       * @var     string
       */
      var $type;

      /**
       * Set method for data type
       *
       * @access public
       * @param string one of bool, int, float, string
       */
      function set_type($type)
      {
          switch ($type) {
          case "bool":
              $this->onupdate = "OnUpdateBool";
              $this->c_type = "zend_bool";
              return true;          

          case "int":
              $this->onupdate = "OnUpdateLong";
              $this->c_type = "long";
              return true;

          case "float":
              $this->onupdate = "OnUpdateReal";
              $this->c_type = "double";
              return true;

          case "string":
              $this->onupdate = "OnUpdateString";
              $this->c_type = "char *";
              return true;

          default:
              return $this->raiseError("'$this->type' not supported, only bool, int, float and string"); 
          }
      }




    /**
     * Directive default value
     *
     * @access private
     * @var     string
     */
    var $value;

    /**
     * Set method for default value
     *
     * @access public
     * @param string default value 
     */
    function set_value($value)
    {
        // TODO checks
        $this->value = $value;
        
        return true;
    }




    /**
     * Directive description
     *
     * @access private
     * @var     string
     */
    var $desc;

    /**
     * Set method for directive description
     *
     * @access public
     * @param string description
     */
    function set_desc($desc)
    {
        $this->desc = $desc;
        
        return true;
    }





    /**
     * Directive access mode
     *
     * @access private
     * @var     string
     */
    var $access = "PHP_INI_ALL";

    /**
     * Set method for access mode
     *
     * @access private
     * @param string access mode specification (system|perdir|user|all)
     */
    function set_access($access)
    {
        switch ($access) {
        case "system":
            $this->access = "PHP_INI_SYSTEM";
            return true;
        case "perdir":
            $this->access = "PHP_INI_PERDIR";
            return true;
        case "user":
            // TODO shouldn't this be ALL instead?
            $this->access = "PHP_INI_USER";
            return true;
        case "all":
        case "":
            $this->access = "PHP_INI_ALL";
            return true;
        default:
            return $this->raiseError("'$access' is not a valid access mode (system|perdir|user|all)");
        }
    }




    /**
     * Directive OnUpdate handler
     *
     * @access private
     * @var     string
     */
    var $onupdate;

    /**
     * Set method for OnUpdate handler
     *
     * @access public
     * @param string C function name
     */
    function set_onupdate($name)
    {
        if (!self::is_name($name)) {
            return $this->raiseError("'$name' is not a valid update function name");
        }
        
        $this->onupdate = $name;
        
        return true;
    }




    /**
     * Internal C type that stores the directives value
     *
     * @access private
     * @var     string
     */
    var $c_type;


    /**
     * Constructor
     */
    function __construct() 
    {
      $this->set_type("string");
    }




    /**
     * Generate header for ini directive registration code
     *
     * @access private
     * @param  string extension basename
     * @return string C code snippet 
     */
    static function c_code_header($name) 
    {
            // this is a small incompatibility between ZE1 and ZE2 APIs
            // "OnUpdateInt" was changed to "OnUpdateLong" as it actualy
            // works on C type "long", not "int"
            // the actual implementation didn't change so it is safe to
            // just revert the name change using a cpp #define
            // TODO: skip this for extensions that depend on PHP 5 anyway
            $code = "
#ifndef ZEND_ENGINE_2
#define OnUpdateLong OnUpdateInt
#endif
";

      $code .="PHP_INI_BEGIN()\n";
            
            return $code;
    }

    /**
     * Generate registration code for this directive
     *
     * @access private
     * @param  string extension basename
     * @return string C code snippet 
     */
    function c_code($name) 
    {
        return "  STD_PHP_INI_ENTRY(\"$name.{$this->name}\", \"{$this->value}\", {$this->access}, {$this->onupdate}, {$this->name}, zend_{$name}_globals, {$name}_globals)\n";
    }

    /**
     * Generate footer for ini directive registration code
     *
     * @access private
     * @param  string extension basename
     * @return string C code snippet 
     */
    static function c_code_footer($name) 
    {
        return "PHP_INI_END()\n\n";
    }



    /**
     * Generate header for ini directive documentation
     *
     * @access private
     * @param  string extension basename
     * @return string DocBook XML snippet 
     */
    static function docbook_xml_header($name) 
    {
        return
"    <table>
     <title>$name runtime configuration</title>
      <tgroup cols='3'>
       <thead>
        <row>
         <entry>directive</entry>
         <entry>default value</entry>
         <entry>descrpition</entry>
        </row>
       </thead>
      <tbody>
";
    }

    /**
     * Generate documentation for ini directive documentation
     *
     * @access private
     * @param  string id basename for extension 
     * @return string DocBook XML snippet 
     */
    function docbook_xml($base) 
    {
        return 
"    <row>
     <entry>$this->name</entry>
     <entry>$this->value</entry>
     <entry>$this->desc</entry>
    </row>
";
    }

    /**
     * Generate footer for ini directive documentation
     *
     * @access private
     * @param  string extension basename
     * @return string DocBook XML snippet 
     */
    static function docbook_xml_footer() 
    {
        return
"     </tbody>
    </tgroup>
   </table>
";
    }
  }

?>
