<?php
//
// +----------------------------------------------------------------------+
// | PHP Version 5                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 3.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/3_0.txt.                                  |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Hartmut Holzgraefe <hholzgra@php.net>                       |
// +----------------------------------------------------------------------+
//
// $Id: Logo.php,v 1.1 2004/04/01 18:43:26 hholzgra Exp $
//

/**
 * Class that manages internal logo images
 *
 * @access public
 */
class PECL_Gen_Element_Logo 
  extends PECL_Gen_Element 
{
    /**
     * Basename
     *
     * @access private
     * @var string
     */
    var $name;
    
    /**
     * ID for URL call of image ("...?=ID")
     *
     * @access private
     * @var string
     */
    var $id;
    
    /**
     * The actual image data as a binary string
     *
     * @access private
     * @var string
     */
    var $data;
    
    /**
     *
     * @access private
     * @var string
     */
    var $mime_type;
    
    /**
     * Constructor
     *
     * @access public
     * @param  string image basename
     * @param  string source filename
     */
    function __construct($name, $src, $mimetype="image/gif") 
    {
        $this->name = $name;
        $this->id = '"'.strtoupper($name).'_LOGO_ID"';
        
        $this->data = file_get_contents($src);
        
        // TODO mimetype should default to false and automatic type detection should be tried here
        $this->mime_type = $mimetype;
    } 
    
    
    /** 
     * Code snippet for image registration in MINFO()
     *
     * @access public
     * @return string C code snippet
     */
    function minit_code() 
    {
        return "  php_register_info_logo({$this->id}, \"{$this->mime_type}\", {$this->name}_logo, ".strlen($this->data).");\n";
    }

    /**
     * Code snippet for image data declaration
     *
     * @access public
     * @param  string extension name
     * @return string C code snippet
     */
    function c_code($name) 
    {
        return "
#include \"php_logos.h\"

static unsigned char {$this->name}_logo[] = {
#include \"{$this->name}_logo.h\"
}; 
";
    }

    /**
     * Generate data for declaration include file
     *
     * @access public
     * @return string C code snippet
     */
    function h_code() 
    {
        $len = strlen($this->data);
        $code = " ";
        $i=0;
        for ($n = 0; $n < $len; $n++) {
            $code .= sprintf(" %3d",ord($this->data[$n]));
            if ($n == $len - 1) break;
            $code .=  ",";
            if (++$i==8) {
                $code .= "\n ";
                $i=0;
            }
        }
        
        $code .= "\n";
        
        return $code;
    }

    /**
     * Documentation snippet (actualy none)
     *
     * @access public
     * @return string empty
     */
    function docbook_xml($base) 
    {
        return "";
    }
    
}

?>