<?php
//
// +----------------------------------------------------------------------+
// | PHP Version 5                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 3.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/3_0.txt.                                  |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Hartmut Holzgraefe <hholzgra@php.net>                       |
// +----------------------------------------------------------------------+
//
// $Id: Resource.php,v 1.2 2004/06/02 13:18:05 hholzgra Exp $
//

/**
 * Class for managing PHP internal resource types
 *
 * @access public
 */
class PECL_Gen_Element_Resource 
    extends PECL_Gen_Element 
{
    /**
     * Resource type name
     *
     * @var string
     * @access private
     */
    var $name = "unknown";

    /**
     * Set method for name
     *
     * @access public
     * @param  string name
     * @return bool   true on success
     */
    function set_name($name) 
    {
        if (!self::is_name($name)) {
            return $this->raiseError("'$name' is not a valid resource name");
        }
        
        $this->name = $name;
        
        return true;
    }
    



    /**
     * Type of the payload that the resource data pointer points to
     *
     * @var string
     * @access private
     */
    var $payload = "void";

    /**
     * Set method for payload type
     *
     * @access public
     * @param  string type name
     * @return bool   true on success
     */
    function set_payload($type)
    {
        $this->payload = $type;
        
        return true;
    }
    



    /**
     * Whether the resource memory is allocated and freed by the extension itself
     *
     * @var string
     * @access private
     */
    var $alloc = "yes";

    /**
     * Set method for alloc
     *
     * @access public
     * @param  string "yes" or "no"
     * @return bool   true on success
     */
    function set_alloc($text)
    {
        // TODO check values
        $this->alloc = $text;
        
        return true;
    }




    /** 
     * Code snippet to be added to the resource destructor callback
     *
     * @var string
     * @access private
     */
    var $destruct = "";

    /**
     * Set method for destructor snippet
     *
     * @access public
     * @param  string C code snippet
     * @return bool   true on success
     */
    function set_destruct($text)
    {
        $this->destruct = $text;
        
        return true;
    }
    



    /**
     * DocBook XML snippet that describes the resource for the manual
     *
     * @var string
     * @access private
     */
    var $description = "";

    /**
     * Set method for destructor snippet
     *
     * @access public
     * @param  string C code snippet
     * @return bool   true on success
     */
    function set_description($text)
    {
        $this->description = $text;
        
        return true;
    }
    

    
    
    /** 
     * Generate resource registration code for MINIT()
     *
     * @access public
     * @return string C code snippet
     */
    function minit_code() {
        return "
le_{$this->name} = zend_register_list_destructors_ex({$this->name}_dtor, 
                                                     NULL, 
                                                     \"{$this->name}\", 
                                                     module_number);

";
    }


    /** 
     * Generate C code for resource destructor callback
     *
     * @access public
     * @param  object extension
     * @return string C code snippet
     */
    function c_code($extension) {
        $dtor = "int le_{$this->name};\n";

        if ($extension->language == "cpp") {
            $dtor.= 'extern "C" ';
        }

		$dtor.= 
"void {$this->name}_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
    {$this->payload} * resource = ({$this->payload} *)(rsrc->ptr);

";

        $dtor .= PECL_Gen_Tools_Indent::indent(4, $this->destruct);

        if ($this->alloc === "yes") {
            $dtor .= "\n\tefree(resource);\n";
        }
        
        $dtor .= "}\n";
        
        return $dtor;
    }



    /** 
     * Generate covenience macros for resource access
     *
     * @access public
     * @return string C code snippet
     */
    function h_code() {
        $upname = strtoupper($this->name);
        
        return "
#define {$upname}_REGISTER(r)   ZEND_REGISTER_RESOURCE(return_value, r, le_{$this->name});
#define {$upname}_FETCH(r, z)   ZEND_FETCH_RESOURCE(r, {$this->payload} *, z, -1, ${$this->name}, le_{$this->name}); if (!r) { RETURN_FALSE; }

";
    }



    /** 
     * Generate documentation for this resource
     *
     * @access public 
     * @param  string id basename for extension
     * @return string DocBook XML code snippet
     */
    function docbook_xml($base) {
        return "
    <section id='$base.resources.{$this->name}'>
     <title><literal>{$this->name}</literal></title>
     <para>
      {$this->description}
     </para>
    </section1
>
";
    }
    
}

?>