<?php
//
// +----------------------------------------------------------------------+
// | PHP Version 5                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 3.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/3_0.txt.                                  |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Hartmut Holzgraefe <hholzgra@php.net>                       |
// +----------------------------------------------------------------------+
//
// $Id: ExtensionParser.php,v 1.4 2004/11/09 18:51:48 hholzgra Exp $
//

    require_once "PECL/Gen/XmlParser.php";
    require_once "PECL/Gen/Maintainer.php";
    require_once "PECL/Gen/Tools/Indent.php";

    /**
     * Class that parses
     *
     * @access public
     */
    class PECL_Gen_ExtensionParser 
        extends PECL_Gen_XmlParser
    {
        /**
         * The extension to parse specs into
         *
         * @access private
         * @var    object
         */
        var $extension;

        /**
         * A helper stack for collecting stuff 
         *
         * @access private
         * @var    array
         */
        var $helper_stack = array();

        /**
         * The current helper (top of stack) 
         *
         * @access private
         * @var    mixed
         */
        var $helper = false;
        
        /** 
         * Constructor
         *
         * @access public
         * @param  object the extension to parse specs into
         */
        function __construct($extension) 
        {
            parent::__construct();
            $this->extension = $extension;
        }

        /**
         * Push something on the helper stack
         *
         * @access private
         * @param mixed
         */
        function push_helper($helper)
        {
            array_push($this->helper_stack, $this->helper);
            $this->helper = $helper;
        }


        /**
         * Pop something from the helper stack
         *
         * @access private
         */
        function pop_helper()
        {
            $this->helper = array_pop($this->helper_stack);
        }

        function tagstart_extension($attr)
        {
            if (!isset($attr["name"])) return $this->raiseError("needed attribute 'name' not given");
            return $this->extension->set_name(trim($attr["name"]));
        }

        function tagstart_extension_name($attr)
        {
            return $this->raiserError("extension name tag is no longer supported, use <extension>s 'name=...' attribute instead");
        }


        function tagend_extension_summary($attr, $data)
        {
            return $this->extension->set_summary(PECL_Gen_Tools_Indent::linetrim($data));
        }

        function tagend_extension_description($attr, $data)
        {
            return $this->extension->set_description(PECL_Gen_Tools_Indent::linetrim($data));
        }

        



        function tagstart_maintainer($attr)
        {
            $this->push_helper(new PECL_Gen_Maintainer);
            return true;
        }

        function tagend_maintainer_user($attr, $data)
        {
            return $this->helper->set_user(trim($data));
        }

        function tagend_maintainer_name($attr, $data)
        {
            return $this->helper->set_name(trim($data));
        }

        function tagend_maintainer_email($attr, $data)
        {
            return $this->helper->set_email(trim($data));
        }

        function tagend_maintainer_role($attr, $data)
        {
            return $this->helper->set_role(trim($data));
        }

        function tagend_maintainer($attr, $data)
        {
            $err = $this->extension->add_author($this->helper);
            $this->pop_helper();
            return $err;
        }

        function tagend_maintainers($attr, $data) {
            return true;
        }



        function tagstart_extension_function($attr) {
            return $this->tagstart_functions_function($attr);
        }
        
        function tagstart_functions_function($attr)
        {
            $this->push_helper(new PECL_Gen_Element_Function);

            if (isset($attr["name"])) {
                $err = $this->helper->set_name($attr["name"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } else {
                return $this->raiseError("name attribut for function missing");
            }

            $role = isset($attr["role"]) ? $attr["role"] : "public";

            $err = $this->helper->set_role($role);
            if (PEAR::isError($err)) {
                return $err;
            }
            
            return true;
        }

        function tagend_function_summary($attr, $data) 
        {
            return $this->helper->set_summary(trim($data));
        }

        function tagend_function_description($attr, $data) 
        {
            return $this->helper->set_description(PECL_Gen_Tools_Indent::linetrim($data));
        }

        function tagend_function_proto($attr, $data)
        {
            return $this->helper->set_proto(trim($data));
        }

        function tagstart_function_code($attr)
        {
            if (isset($attr["src"])) {
                if (!file_exists($attr["src"])) {
                    return $this->raiseError("Soruce file '$attr[src]' not found");                    
                }
                if (!is_readable($attr["src"])) {
                    return $this->raiseError("Cannot read source file '$attr[src]'");                    
                }
            }
        }

        function tagend_function_code($attr, $data)
        {
            if (isset($attr["src"])) {
                return $this->helper->set_code(PECL_Gen_Tools_Indent::linetrim(file_get_contents($attr["src"])));
            } else {
                return $this->helper->set_code(PECL_Gen_Tools_Indent::linetrim($data));
            }
        }

        function tagend_extension_function($attr, $data)
        {
            return $this->tagend_functions_function($attr, $data);
        }

        function tagend_functions_function($attr, $data) 
        {
            $err = $this->extension->add_function($this->helper);
            $this->pop_helper();
            return $err;
        }


        function tagend_functions($attr, $data) {
            return true;
        }        


        function tagstart_extension_resource($attr)
        {
            return $this->tagstart_resources_resource($attr);
        }

        function tagstart_resources_resource($attr)
        {
            $this->push_helper(new PECL_Gen_Element_Resource);
            
            if (isset($attr["name"])) {
                $err = $this->helper->set_name($attr["name"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } else {
                return $this->raiseError("name attribut for resource missing");
            }

            if (isset($attr["payload"])) {
                $err = $this->helper->set_payload($attr["payload"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } else {
                return $this->raiseError("payload attribut for resource missing");
            }

            if (isset($attr["alloc"])) {
                $err = $this->helper->set_alloc($attr["alloc"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } 

            return true;
        }

        function tagend_resource_destruct($attr, $data)
        {
            return $this->helper->set_destruct(PECL_Gen_Tools_Indent::linetrim($data));
        }

        function tagend_resource_description($attr, $data)
        {
            return $this->helper->set_description(PECL_Gen_Tools_Indent::linetrim($data));
        }

        function tagend_extension_resource($attr, $data) {
            return $this->tagend_resources_resource($attr, $data);
        }

        function tagend_resources_resource($attr, $data) 
        {
            $err = $this->extension->add_resource($this->helper);
            $this->pop_helper();
            return $err;
        }


        function tagend_resources($attr, $data) {
            return true;
        }



        function tagend_extension_logo($attr, $data)
        {
            // TODO checks
            $logo = new PECL_Gen_Element_Logo($this->extension->name, $attr["src"], $attr["mimetype"]);
            return $this->extension->set_logo($logo);
        }



        function tagend_extension_constant($attr, $data)
        {
            return $this->tagend_constants_constant($attr, $data);
        }

        function tagend_constants_constant($attr, $data)
        {
            $const = new PECL_Gen_Element_Constant;

            if (isset($attr["name"])) {
                $err = $const->set_name($attr["name"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } else {
                return $this->raiseError("name attribut for constant missing");
            }

            if (isset($attr["type"])) {
                $err = $const->set_type($attr["type"]);
            } else {
                $err = $const->set_type("int"); // default
            }
            if (PEAR::isError($err)) {
                return $err;
            }

            if (isset($attr["value"])) {
                $err = $const->set_value($attr["value"]);
            } else {
                $err = $const->set_value($attr["name"]); // default -> mimic a C #define or enum value
            } 
            if (PEAR::isError($err)) {
                return $err;
            }
            
            $const->set_desc(PECL_Gen_Tools_Indent::linetrim($data));


            return $this->extension->add_constant($const);
        }

        function tagend_constants($attr, $data) {
            return true;
        }


        
        function tagend_extension_global($attr, $data)
        {
            return $this->tagend_globals_global($attr, $data);
        }


        function tagend_globals_global($attr, $data)
        {
            $global = new PECL_Gen_Element_Global;

            if (isset($attr["name"])) {
                $err = $global->set_name($attr["name"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } else {
                return $this->raiseError("name attribut for global missing");
            }

            if (isset($attr["type"])) {
                $err = $global->set_type($attr["type"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } else {
                return $this->raiseError("type attribut for global missing");
            }

            if (isset($attr["value"])) {
                $err = $global->set_value($attr["value"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } 

            return $this->extension->add_global($global);
        }

        

        function tagend_extension_phpini($attr, $data) 
        {
            return $this->tagend_globals_phpini($attr, $data);
        }

        function tagend_globals_phpini($attr, $data)
        {
            $ini = new PECL_Gen_Element_Ini;

            if (isset($attr["name"])) {
                $err = $ini->set_name($attr["name"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } else {
                return $this->raiseError("name attribut for php.ini directive missing");
            }

            if (isset($attr["type"])) {
                $err = $ini->set_type($attr["type"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } 

            if (isset($attr["value"])) {
                $err = $ini->set_value($attr["value"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } 

            if (isset($attr["access"])) {
                $err = $ini->set_access($attr["access"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } 

            if (isset($attr["onupdate"])) {
                $err = $ini->set_onupdate($attr["onupdate"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } 

            $ini->set_desc(PECL_Gen_Tools_Indent::linetrim($data));

            $err = $this->extension->add_phpini($ini);
            if (PEAR::isError($err)) {
                return $err;
            }
            
            // php.ini settings are stored in modul-global variables
            $global = new PECL_Gen_Element_Global;
            $err = $global->set_name($ini->name);
            if (PEAR::isError($err)) {
                return $err;
            }
            $err = $global->set_type($ini->c_type);
            if (PEAR::isError($err)) {
                return $err;
            }
            $err = $global->set_value($ini->value);
            if (PEAR::isError($err)) {
                return $err;
            }
            
            $err = $this->extension->add_global($global);

            return $err;
        }

        function tagend_globals($attr, $data) {
            return true;
        }
        


        function tagstart_extension_release($attr)
        {
            $this->push_helper(new PECL_Gen_Release);
            return true;
        }

        function tagend_release_version($attr, $data)
        {
            return $this->helper->set_version(trim($data));
        }

        function tagend_release_date($attr, $data)
        {
            return $this->helper->set_date(trim($data));
        }

        function tagend_release_state($attr, $data)
        {
            return $this->helper->set_state(trim($data));
        }

        function tagend_release_notes($attr, $data)
        {
            return $this->helper->set_notes(PECL_Gen_Tools_Indent::linetrim($data));
        }

        function tagend_extension_release($attr, $data)
        {
            $err = $this->extension->set_release($this->helper);
            $this->pop_helper(new PECL_Gen_Release);
            return $err;
        }


        
        function tagend_license($attr, $data)
        {
          $license = PECL_Gen_License::factory(trim($data));

            if (PEAR::isError($license)) {
                return $err;
            }

            return $this->extension->set_license($license);
        }



        function tagstart_extension_deps($attr)
        {
            if (isset($attr["language"])) {
                $err = $this->extension->set_language($attr["language"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            }

            if (isset($attr["platform"])) {
                $err = $this->extension->set_platform($attr["platform"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            }
        }

        function tagstart_deps_lib($attr)
        {
            $this->extension->libs[$attr['name']] = $attr;
            if (isset($attr['platform'])) {
                $platform = new PECL_Gen_Tools_Platform($attr["platform"]);
            } else {
                $platform = new PECL_Gen_Tools_Platform("all");
            }

            if (PEAR::isError($platform)) {
                return $platform;
            }

            $this->extension->libs[$attr['name']]['platform'] = $platform;
            return true;
        }

        function tagstart_deps_header($attr)
        {
            $this->extension->headers[$attr['name']] = $attr; 
        }

        function tagend_deps_with($attr, $data) {
            $this->extension->with = array();
            $this->extension->with['attr'] = $attr;
            $this->extension->with['desc'] = trim($data);       
        }


        function tagstart_deps_file($attr, $data) {
            if (!isset($attr['name'])) {
                return $this->raiseError("name attribut for file missing");
            }

            return $this->extension->add_source_file($attr['name']);
        }


        function tagstart_extension_code($attr)
        {
            if (isset($attr["src"])) {
                if (!file_exists($attr["src"])) {
                    return $this->raiseError("Soruce file '$attr[src]' not found");                    
                }
                if (!is_readable($attr["src"])) {
                    return $this->raiseError("Cannot read source file '$attr[src]'");                    
                }
            }
        }

        function tagend_extension_code($attr, $data) {
            $role     = isset($attr["role"])     ? $attr["role"]     : "code";
            $position = isset($attr["position"]) ? $attr["position"] : "bottom";

            if (isset($attr["src"])) {
                return $this->extension->add_code($role, $position, PECL_Gen_Tools_Indent::linetrim(file_get_contents($attr["src"])));
            } else {
                return $this->extension->add_code($role, $position, PECL_Gen_Tools_Indent::linetrim($data));
            }
        }

        function tagend_extension_makefile($attr, $data) {
            return $this->extension->add_makefragment(PECL_Gen_Tools_Indent::linetrim($data));
        }

        function tagend_extension_deps_configm4($attr, $data) {
            return $this->extension->add_configfragment(PECL_Gen_Tools_Indent::linetrim($data), 
                                                        isset($attr['position']) ? $attr['position'] : "top");
        }


        function tagstart_test($attr) {
            $test = new PECL_Gen_Element_Test();

            if (isset($attr["name"])) {
                $err = $test->set_name($attr["name"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } else {
                return $this->raiseError("name attribut for class missing");
            }
            $this->push_helper($test);
        }

        function tagend_test_title($attr, $data) {
            $this->helper->set_title(PECL_Gen_Tools_Indent::linetrim($data));
        }

        function tagend_test_skipif($attr, $data) {
            $this->helper->set_skipif(PECL_Gen_Tools_Indent::linetrim($data));
        }

        function tagend_test_get($attr, $data) {
            $this->helper->set_get(PECL_Gen_Tools_Indent::linetrim($data));
        }

        function tagend_test_post($attr, $data) {
            $this->helper->set_post(PECL_Gen_Tools_Indent::linetrim($data));
        }

        function tagstart_test_code($attr)
        {
            if (isset($attr["src"])) {
                if (!file_exists($attr["src"])) {
                    return $this->raiseError("Soruce file '$attr[src]' not found");                    
                }
                if (!is_readable($attr["src"])) {
                    return $this->raiseError("Cannot read source file '$attr[src]'");                    
                }
            }
        }

        function tagend_test_code($attr, $data) {
            if (isset($attr["src"])) {
                $this->helper->set_code(PECL_Gen_Tools_Indent::linetrim(file_get_contents($attr["src"])));
            } else {
                $this->helper->set_code(PECL_Gen_Tools_Indent::linetrim($data));
            }
        }

        function tagend_test_output($attr, $data) {
            $this->helper->set_output(PECL_Gen_Tools_Indent::linetrim($data));
        }

        function tagend_test($attr, $data) {
            $err = $this->extension->add_test($this->helper);
            $this->pop_helper();
            return $err;
        }

        function tagend_tests($attr, $data) {
            return true;
        }

        function tagend_changelog($attr, $data) {
            return true;
        }





        function tagstart_class($attr)
        {
            $this->push_helper(new PECL_Gen_Element_Class);

            if (isset($attr["name"])) {
                $err = $this->helper->set_name($attr["name"]);
                if (PEAR::isError($err)) {
                    return $err;
                }
            } else {
                return $this->raiseError("name attribut for class missing");
            }

            if (PEAR::isError($err)) {
                return $err;
            }
            
            return true;
        }

        function tagend_class_summary($attr, $data) 
        {
            return $this->helper->set_summary(trim($data));
        }

        function tagend_class_description($attr, $data) 
        {
            return $this->helper->set_description(PECL_Gen_Tools_Indent::linetrim($data));
        }

        function tagstart_class_code($attr)
        {
            if (isset($attr["src"])) {
                if (!file_exists($attr["src"])) {
                    return $this->raiseError("Soruce file '$attr[src]' not found");                    
                }
                if (!is_readable($attr["src"])) {
                    return $this->raiseError("Cannot read source file '$attr[src]'");                    
                }
            }
        }

        function tagend_class_code($attr, $data)
        {
            if (isset($attr["src"])) {
                return $this->helper->set_code(PECL_Gen_Tools_Indent::linetrim(file_get_contents($attr["src"])));
            } else {
                return $this->helper->set_code(PECL_Gen_Tools_Indent::linetrim($data));
            }
        }

        function tagend_class($attr, $data) 
        {
            $err = $this->extension->add_class($this->helper);
            $this->pop_helper();
            return true;
        }

    }


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * indent-tabs-mode:nil
 * End:
 */
?>
