<?php
//
// +----------------------------------------------------------------------+
// | PHP Version 5                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 3.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/3_0.txt.                                  |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Hartmut Holzgraefe <hholzgra@php.net>                       |
// +----------------------------------------------------------------------+
//
// $Id: License.php,v 1.3 2004/06/02 13:18:51 hholzgra Exp $
//

/**
* Abstract base class for license handling
* 
* @version $Id: License.php,v 1.3 2004/06/02 13:18:51 hholzgra Exp $
*/

abstract class PECL_Gen_License extends PEAR
{
    /**
     * Constructor
     *
     * @access private
     * @param  License options
     */
    function __construct($options = array()) 
    {
        $this->options = $options;
    }

    /**
     * Takes a License shortname and returns an instantiated object of that license
     *
     * @access  public
     * @param   string  License shortname, e.g. PHP, BSD, LGPL
     * @param   array   License options
     * @returns object  License instance
     */
    static function factory($name, $options=array()) 
    {
        $classname = "PECL_Gen_License_".strtoupper($name);
        $classfile = str_replace("_", "/", $classname).".php";

        if (!class_exists($classname)) {
            if (!@include_once "$classfile")
                $this->raiseError("Unknown license type '$name' ", E_USER_WARNING);
        }

        return 
            class_exists($classname) 
            ? new $classname($options) 
            : false;
    }

    /**
     * Writes the License text to a file
     *
     * @param   string Filename to write to (default is ./LICENSE) 
     * @return  bool   Success state
     * @access  public
     */
    function writeToFile($path = "./LICENSE") 
    {
        $fp = fopen($path, "w");

        if (is_resource($fp)) {
            fputs($fp, $this->getText()); 
            fclose($fp);
            return true;
        }

        return false;
    }
    
    /**
     * Returns a string suitable for insertion into comment headers
     * 
     * @return string    License comment
     * @access public
     */
    abstract function getComment();

    /**
     * Returns the complete license text as string
     * 
     * @return string    License text
     * @access public
     */
    abstract function getText();
}

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * indent-tabs-mode:nil
 * End:
 */

?>