<?php
//
// +----------------------------------------------------------------------+
// | PHP Version 5                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 3.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/3_0.txt.                                  |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Hartmut Holzgraefe <hholzgra@php.net>                       |
// +----------------------------------------------------------------------+
//
// $Id: Indent.php,v 1.1 2004/04/01 18:43:26 hholzgra Exp $
//


/**
 * Wrapper class for whitespace related stuff
 *
 * all methods are actually static, the class is just needed for
 * namespace emulation to conform with PEAR naming conventions
 */

class PECL_Gen_Tools_Indent {
    /**
     * Replace leading blanks with tabs
     *
     * PHP C coding conventions require tab indention 
     * with a tabsize of 4
     *
     * @access public
     * @param  string text to tabify
     * @param  int    tab indention level (default: 4 spaces)
     * @return string tabified text
     */
    function tabify($text, $tabsize = 4) 
    {
        return preg_replace_callback("/^(".str_repeat(" ", $tabsize).")+/m", 
            create_function('$matches','return str_repeat("\t", strlen($matches[0])/'.$tabsize.');'), 
            $text
        );
    }


    /**
     * Replace leading tabs with blanks
     *
     * PEAR PHP coding conventions require blank indention 
     * with a tabsize of 4
     *
     * @access public
     * @param  string text to untabify
     * @param  int    tab indention level (default: 4 spaces)
     * @return string untabified text
     */
    function untabify($text, $tabsize = 4) 
    {
        return preg_replace_callback("/^(\t)*/m", 
            create_function('$matches','return str_repeat(" ", strlen($matches[0])*'.$tabsize.');'), 
            $text
        );
    }

	/**
	 * re-indent a block of text
	 *
	 * @access public
	 * @param  int    number of leading indent spaces
	 * @param  string text to reindent
	 * @return string indented text
	 */
    function indent($level, &$text) 
    {
        $lines = explode("\n", $text);

        while (count($lines) && trim(end($lines)) == "") {
            array_pop($lines);
        }

        $maxindent = 999;
        foreach ($lines as $line) {
            if (trim($line)=="") continue;
            ereg("^ *", $line, $matches);
            $maxindent = min($maxindent, strlen($matches[0]));
        }

        $result = "";
        $find = str_repeat(" ", $maxindent);
        $replace = str_repeat(" ", $level);
        foreach ($lines as $line) {
            $result.= self::tabify(ereg_replace("^$find", $replace, $line)."\n", 4);
        }

        return $result;
    }


    /**
     * Trim of leading and trailing whitespace-only lines
     *
     * @access public
     * @param  string  text
     * @return string  trimmed text
	 */
    function linetrim($text)
    {
        $text = preg_replace('|^\s*\n|','', $text);
        $text = preg_replace('|\n\s*$|',"\n", $text);
        return $text;
    }
}

?>