<?php
//
// +----------------------------------------------------------------------+
// | PHP Version 5                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 3.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/3_0.txt.                                  |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Hartmut Holzgraefe <hholzgra@php.net>                       |
// +----------------------------------------------------------------------+
//
// $Id: Platform.php,v 1.1 2004/04/01 18:43:26 hholzgra Exp $
//

/**
 * A helper class for platform management
 *
 * @access public
 */
class PECL_Gen_Tools_Platform
    extends PEAR
{
    /**
     * The supported platforms are collected here
     *
     * @access private
     * @var    array   the platform short names
     */
    var $platforms = array();

    /**
     * Constructor gets a list of names or "all"
     *
     * @access public
     * @param  string|array names as comma separated string or array
     */
    function __construct($names) 
    {
        if (is_string($names)) {
            $names = explode(",", $names);
        }
        
        foreach ($names as $name) {
            switch (strtolower(trim($name))) {
            case "all":
                $this->platforms["win"] = true;
                $this->platforms["unix"] = true;
                break;

            case "win":
            case "win23":
            case "windows":
            case "microsoft":
                $this->platforms["win"] = true;
                break;

            case "unix":
            case "posix":
            case "gnu":
                $this->platforms["unix"] = true;
                break;

            default:
                $this->error = $this->raiseError("'$name' is not a supported platform");
                break(2);
            }
        }
    }

    /**
     * Test for a platform shortname
     *
     * @access public
     * @param  string shortname
     * @return bool   true if supported else false
     */
    function test($name) 
    {
        switch (strtolower(trim($name))) {
        case "all":
            return 2 == $this->count(); 
            
        case "win":
        case "win23":
        case "windows":
        case "microsoft":
            return isset($this->platforms["win"]);
            
        case "unix":
        case "posix":
        case "gnu":
            return ($this->platforms["unix"]);
            
        default:
            return false;
        }
    }

    /** 
     * Count the number of supported platforms
     *
     * @access public
     * @return int    platform count
     */
    function count()
    {
        return count($this->platforms());
    }
}

?>